mNameplates = LibStub("AceAddon-3.0"):NewAddon("mNameplates", "AceEvent-3.0", "AceConsole-3.0")
local mNameplates = mNameplates
MDB_MN = mNameplates


local UIF
local LSM = LibStub("LibSharedMedia-3.0")
local AceGUI = LibStub("AceGUI-3.0")
local Nameplates = LibStub("LibNameplate-1.0")

local db

local bossIconTexture = [[Interface\TargetingFrame\UI-TargetingFrame-Skull]]

local ResetTextures = {
	ThreatGlow = [[Interface\TargetingFrame\UI-TargetingFrame-Flash]],
	Highlight = [[Interface\Tooltips\Nameplate-Glow]],
	BossIcon = [[Interface\TargetingFrame\UI-TargetingFrame-Skull]],
	ElitIcon = [[Interface\Tooltips\EliteNameplateIcon]],
	CastBorder = [[Interface\Tooltips\Nameplate-Border]],
	RaidIcon = [[nterface\TargetingFrame\UI-RaidTargetingIcons]],
}

local DefaultWidth, DefaultHeight
local InCombat = false

-- Combat Wrappers
local StateScriptSet = {}
local ApplyPlateSet = {}
local PlateReset = {}

local StateHeader = CreateFrame("Frame", nil, nil, "SecureHandlerBaseTemplate")
StateHeader:RegisterEvent("PLAYER_REGEN_ENABLED")
StateHeader:SetScript("OnEvent", function(self, event, ...)
	for plate in pairs(PlateReset) do
		mNameplates:ResetLockedAttributes(plate)
	end
	wipe(PlateReset)
end)

function mNameplates:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			
			Width = 150,
			Height = 30,
		}
	}
	
	self.db  = LibStub("AceDB-3.0"):New("mNameplatesDB", defaults, "Default")
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	
	self:SetupLDB()
	
	self:SetEnabledState(db.Enabled)
	
	self.Callbacks = LibStub("CallbackHandler-1.0"):New(self)	
	
	self.Plates = {}

	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("mNameplates", {font = LibStub("LibSharedMedia-3.0"):Fetch("font", "accid"), fontSize = 16})
	UIF:SetValuesChangedCallback(function() 
		mNameplates:ToggleConfig(true)
	end)
end

function mNameplates:OnEnable()		
	if UnitAffectingCombat("plater") then InCombat = true end

	self:RegisterChatCommand("mn", "ToggleConfig")
	
	Nameplates.RegisterCallback(self, "LibNameplate_FoundGUID")
	Nameplates.RegisterCallback(self, "LibNameplate_NewNameplate")
	Nameplates.RegisterCallback(self, "LibNameplate_MouseoverNameplate")
	Nameplates.RegisterCallback(self, "LibNameplate_HealthChange")
	
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	
	for i, plate in pairs({select(2, Nameplates:GetAllNameplates())}) do
		self:LibNameplate_NewNameplate("LibNameplate_NewNameplate", plate)
	end
	
	wipe(PlateReset)
end

function mNameplates:OnDisable()
	for moduleName, module in self:IterateModules() do
		LibStub("AceAddon-3.0"):DisableAddon(module)
	end
	for	i, plate in pairs(self.Plates) do
		plate.DefaultItems.ThreatGlow:SetTexture(ResetTextures.ThreatGlow)
		plate.DefaultItems.Highlight:SetTexture(ResetTextures.Highlight)
		plate.DefaultItems.BossIcon:SetTexture(ResetTextures.BossIcon)
		plate.DefaultItems.ElitIcon:SetTexture(ResetTextures.ElitIcon)
		plate.DefaultItems.CastBorder:SetTexture(ResetTextures.CastBorder)
		plate.DefaultItems.RaidIcon:SetTexture(ResetTextures.RaidIcon)
		
		plate.DefaultItems.Name:Show()
		plate.DefaultItems.HealthBorder:Show()
		plate.DefaultItems.Level:Show()
		plate.DefaultItems.HealthBar:Show()
		plate.DefaultItems.CastBar:Show()
		
		plate.DefaultItems = nil
		plate.mn = nil
		
		self:CombatWrapperResetPlate(plate)
	end
	wipe(self.Plates)
	
	Nameplates.UnregisterAllCallbacks(self)
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Callbacks ~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
function mNameplates:LibNameplate_NewNameplate(event, plate)
	self:PrepPlate(plate)
	plate.DefaultItems.Level:Hide()
	
	self:ApplyOnPlate(plate)	
	self.Callbacks:Fire("PlateShow", plate)
end

function mNameplates:LibNameplate_FoundGUID(event, plate, gUID, unit)	
	self.Callbacks:Fire("PlateGUID", gUID, unit)
end

function mNameplates:LibNameplate_MouseoverNameplate(event, plate)
	self.Callbacks:Fire("PlateMouseover", plate)
end

function mNameplates:LibNameplate_HealthChange(event, plate)
	self.Callbacks:Fire("PlateHPChanged", plate)
end

function mNameplates:ProfileChanged(event)
	db = self.db.profile
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Events ~~~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
function mNameplates:UPDATE_MOUSEOVER_UNIT(...)
	local plate = Nameplates:GetNameplateByUnit("mouseover")
	if plate then
		self.Callbacks:Fire("PlateMouuseOver", plate)
	end
end

function mNameplates:PLAYER_REGEN_DISABLED()
	InCombat = true
end

function mNameplates:PLAYER_REGEN_ENABLED()
	InCombat = false
	for plate in pairs(StateScriptSet) do		
		self:SetStateScript(plate)
	end
	for plate in pairs(ApplyPlateSet) do
		self:SetLockedAttributes(plate)
	end
	
	
	wipe(StateScriptSet)
	wipe(ApplyPlateSet)
	wipe(PlateReset)
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Creation ~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
do
	local regions = {}
	function mNameplates:PrepPlate(plate)	
		if plate.mn then return end
		local healthBar, castBar = plate:GetChildren()
		local threatGlow, healthBorder, castbarBorder, castNoInterupt, castIcon, highlight, name, level, bossIcon, raidIcon, eliteIcon = plate:GetRegions()		
		healthBar:Hide()
		castBar:Hide()
		
		threatGlow:SetTexture(nil)
		highlight:SetTexture(nil)
		bossIcon:SetTexture(nil)
		eliteIcon:SetTexture(nil)
		castbarBorder:SetTexture(nil)
		castNoInterupt:SetTexture(nil)		
		raidIcon:SetTexture(nil)
		
		name:Hide()		
		healthBorder:Hide()
		
		plate.mn = {}
		
		plate.DefaultItems = {
			HealthBar = healthBar,
			CastBar = castBar,
			ThreatGlow = threatGlow,
			HealthBorder = healthBorder,
			CastBorder = castbarBorder,
			CastNoInterupt = castNoInterupt,
			Icon = castIcon,
			Highlight = highlight,
			Name = name,
			Level = level,
			BossIcon = bossIcon,
			RaidIcon = raidIcon,
			ElitIcon = eliteIcon,
		}	

		tinsert(self.Plates, plate)
		
		local t = plate:CreateTexture()
		t:SetAllPoints()
		t:SetTexture(0, 0, 0, 0.5)
		DefaultHeight = DefaultHeight or plate:GetHeight()
		DefaultWidth = DefaultWidth or plate:GetWidth()				
		
		self:CombatWrapperSetStateScript(plate)
		
		self.Callbacks:Fire("PlateNew", plate)
	end
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Settings ~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
function mNameplates:ApplyOnPlate(plate)
	self:CombatWrapperApplyPlate(plate)
end

function mNameplates:Applyettings()
	SetCVar("nameplateShowEnemies", 1)
	--SetCVar("nameplateShowEnemies", 0)
	for i, plate in self:IteratePlates() do
		self:ApplyOnPlate(plate)
	end
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Misc ~~~~~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
function mNameplates:IteratePlates()
	return pairs(self.Plates)
end

function mNameplates:GetExtra(plate)
	return plate.mn
end

do 
	local AttachList = {}
	local AnchorList = {}
	function mNameplates:GetAttachList()
		wipe(AttachList)
		for moduleName, module in self:IterateModules() do
			AttachList[moduleName] = module.DisplayName or moduleName
		end
		AttachList["NAMEPLAETE"] = "Name plate"
		
		return AttachList
	end
	
	function mNameplates:GetAttachFrame(moduleName, plate)
		local module = self:GetModule(moduleName, "true")
		if module then
			return module:GetAttachFrame(plate)
		end
	end
	
	function mNameplates:GetAnchorList()		
		AnchorList = AnchorList or {}
		AnchorList["TOPLEFT"] = "Top left"
		AnchorList["TOP"] = "Top"
		AnchorList["TOPRIGHT"] = "Top Right"
		AnchorList["RIGHT"] = "Right"
		AnchorList["BOTTOMRIGHT"] = "Bottom right"
		AnchorList["BOTTOM"] = "Bottom"
		AnchorList["BOTTOMLEFT"] = "Bottom left"
		AnchorList["LEFT"] = "Left"
		AnchorList["CENTER"] = "Cetner"
		
		return AnchorList
	end
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Combat Wrappers ~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~

function  mNameplates:CombatWrapperSetStateScript(plate)
	if InCombat then
		StateScriptSet[plate] = true
	else
		self:SetStateScript(plate)
	end
end

function mNameplates:CombatWrapperApplyPlate(plate)
	if InCombat then
		ApplyPlateSet[plate] = true
	else
		self:SetLockedAttributes(plate)
	end
end

function mNameplates:CombatWrapperResetPlate(plate)
	if InCombat then
		PlateReset[plate] = true
	else
		self:ResetLockedAttributes(plate)
	end
end

function mNameplates:SetStateScript(plate)
	StateHeader:SetFrameRef("TheWorld", WorldFrame)
	StateHeader:WrapScript(plate, "OnShow", [[
		self:SetWidth(]]..db.Width..[[)
		self:SetHeight(]]..db.Height..[[)
	]])
end

function mNameplates:SetLockedAttributes(plate)
	plate:SetWidth(db.Width)
	plate:SetHeight(db.Height)
end

function mNameplates:ResetLockedAttributes(plate)
	plate:SetWidth(DefaultWidth)
	plate:SetHeight(DefaultHeight)
	StateHeader:UnwrapScript(plate, "OnShow")
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Config ~~~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~
do	
	local function Callback()
		mNameplates:Applyettings()
	end

	local OptionsFrame
	local SelectedOption
	local OptionsStatus = {}
	
	function mNameplates:ToggleConfig(reOpen)
		if not OptionsFrame then	
			OptionsFrame = UIF:Frame("mNameplates")
			OptionsFrame:SetLayout("Fill")
			OptionsFrame:SetWidth(813)
			OptionsFrame:SetHeight(630)
			OptionsFrame:Hide()			
			OptionsFrame:SetCallback("OnClose", function(widget)				
				OptionsFrame:ReleaseChildren()		
				OptionsFrame.frame:Hide()
			end)			
		end
		
		if OptionsFrame.frame:IsShown() then
			OptionsFrame:Hide()
			if not reOpen then
				return
			end
		end
		
		m = UIF:TreeGroup()
		m:SetLayout("Fill")	
		m:SetCallback("OnGroupSelected", function(self, event, group) SelectedOption = group end)									
		OptionsFrame:AddChild(m)
		
		m:AddGroup({{text = "Main", value = "Main", order = 0}}, function()
			local g = UIF:InlineGroup1()
			
			g:AddChild(UIF:CheckBox("Enabled", db, "Enabled", function() 
				if db.Enabled and not mNameplates:IsEnabled() then mNameplates:Enable()
				elseif not db.Enabled and mNameplates:IsEnabled() then mNameplates:Disable() end
			end))
			
			g:AddChild(UIF:Slider("Width", db, "Width", 0, 300, 0.1, Callback, 0.5))
			g:AddChild(UIF:Slider("Height", db, "Height", 0, 300, 0.1, Callback, 0.5))
			
			return g
		end)
	
		for moduleName, module in self:IterateModules() do
			if module.GetOptions then
				local options = module:GetOptions()
				if options and type(options) == "table" then
					for i, option in pairs(options) do
						m:AddGroup(option.data, option.func)
					end
				end
			end
		end
		
		m:AddGroup({{text = "Profile", value = "Profile"}}, function()
			return UIF:AceProfileOption(mNameplates.db)
		end)
		
		m:SetSelected(SelectedOption or "Main")
		m:SetStatusTable(OptionsStatus)	
		OptionsFrame:Show()
	end

	function mNameplates:SetupLDB()
		local LDB = LibStub("LibDataBroker-1.1")
		if not LDB then return end
		
		local l = LDB:NewDataObject("mNameplates")
		l.type = "launcher"
		l.icon = [[Interface\Icons\Spell_DeathKnight_RuneTap]]
		l.tocname = "mNameplates"
		l.label = "mNameplates"
		l.OnClick = function(self, button)
			mNameplates:ToggleConfig()
		end
		l.OnTooltipShow = function(tt)
			tt:AddLine("mNameplates")
			tt:AddLine("Click to toggle configuration")
		end
		
		self.ldbojb = l
	end
end